use std::fmt::Display;

use crate::constants::HIGHLIGHT_COLOR;
use tui::buffer::Buffer;
use tui::layout::Rect;
use tui::style::{Color, Style};
use tui::widgets::*;

use crate::ui_screens::utils::centered_rect_size;

pub struct ButtonWidget {
    is_hovered: bool,
    label: String,
    disabled: bool,
    min_width: usize,
}

impl ButtonWidget {
    pub fn new<D: Display>(label: D, is_hovered: bool) -> Self {
        Self {
            label: label.to_string(),
            is_hovered,
            disabled: false,
            min_width: 0,
        }
    }

    pub fn set_disabled(mut self, disabled: bool) -> Self {
        self.disabled = disabled;
        self
    }

    pub fn set_min_width(mut self, min_width: usize) -> Self {
        self.min_width = min_width;
        self
    }

    pub fn estimated_size(&self) -> (u16, u16) {
        ((self.label.len() + 2).max(self.min_width) as u16, 1)
    }
}

impl Widget for ButtonWidget {
    fn render(self, area: Rect, buf: &mut Buffer) {
        let expected_len = self.estimated_size().0;

        let mut label = self.label.clone();
        while label.len() < expected_len as usize {
            label.insert(0, ' ');
            label.push(' ');
        }

        let area = centered_rect_size(label.len() as u16, 1, &area);

        let input = Paragraph::new(label.as_ref()).style(match (self.disabled, self.is_hovered) {
            (true, _) => Style::default(),
            (_, false) => Style::default().bg(Color::DarkGray),
            (_, true) => Style::default().fg(Color::White).bg(HIGHLIGHT_COLOR),
        });

        input.render(area, buf);
    }
}
